<!DOCTYPE html>
<html lang="en">

<head>
    <style>
        .incident-table {
            table, th, td {
              border: 1px solid black;
            }
        }
    </style>

</head>
<body>
<div style="text-align: left">
{#    <p>{{ evaluation_date_sub_incidents.count }} <span style="color: {{ threshold.severity.color }};">{{ threshold.severity.name }}</span> incidents: </p>#}
    {% if rule.description %}
    <p style="font-style: italic">{{ rule.description }}</p>
    {% endif %}
    <div style="padding: 5px">
    <table class="incident-table" style="width: 100%; text-align: center; border: 1px solid" >
      <tr style="font-size: small;">
        <th colspan="2" style="font-weight: bold; min-width:150px">Breach Info</th>
        <th rowspan="2">{{ check_object_content_type_name }}</th>
        <th rowspan="2" style="min-width:150px">Details</th>
      </tr>
    <tr style="font-size: small;">
        <th style="font-weight: bold">{{ breached_content_type_name }}</th>
        <th style="font-weight: bold">Value</th>
    </tr>
    {% for sub_incident in evaluation_date_sub_incidents %}
        <tr style="font-size: x-small;">
            <td>{{ sub_incident.incident.breached_object_repr }}</td>
            <td style="font-weight: bold">{{ sub_incident.breached_value | safe }}</td>
            <td>{{ sub_incident.rule_check.checked_object_repr }}</td>
            <td>{{ sub_incident.report | safe }}</td>
        </tr>
    {% endfor %}
    </table>
    </div>
</div>
</body>
