from .compliance_form import (
    ComplianceFormAdmin,
    ComplianceFormRuleAdmin,
    ComplianceFormSectionAdmin,
    ComplianceFormSignatureAdmin,
    ComplianceFormSignatureRuleAdmin,
    ComplianceFormSignatureSectionAdmin,
    ComplianceFormTypeAdmin,
)
from .compliance_task import (
    ComplianceActionAdmin,
    ComplianceEventAdmin,
    ReviewComplianceTaskAdmin,
)
from .compliance_type import ComplianceTypeAdmin
from .risk_management import *
