# Generated by Django 5.0.8 on 2024-08-28 11:32

import django.core.serializers.json
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("wbcompliance", "0015_auto_20240103_0957"),
    ]

    operations = [
        migrations.AddField(
            model_name="checkedobjectincidentrelationship",
            name="report_details",
            field=models.JSONField(default=dict, blank=True, encoder=django.core.serializers.json.DjangoJSONEncoder),
        ),
        migrations.AddField(
            model_name="rulebackend",
            name="incident_report_template",
            field=models.TextField(
                default="<ul>\n    {% for key, value in report_details.items %}\n        <li><strong>{{ key }}: </strong> {{ value }}</li>\n    {% endfor %}\n</ul>\n",
                verbose_name="Incident Template",
            ),
        ),
        migrations.AddField(
            model_name="checkedobjectincidentrelationship",
            name="breached_value",
            field=models.CharField(
                blank=True,
                help_text="The value that breached the rule threshold, can be None",
                max_length=128,
                null=True,
                verbose_name="Breached Value",
            ),
        ),
    ]
